@@ -2,15 +2,17 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 3)
+__version__ = (0, 0, 4)
 __all__ = [
-    "HashObj", "make_accumulator", "file_digest", "file_mdigest", 
+    "HashObj", "ChunkedHash", "make_accumulator", "file_digest", "file_mdigest", 
     "file_digest_async", "file_mdigest_async", 
 ]
 
-from collections.abc import AsyncIterable, AsyncIterator, Awaitable, Callable, Iterable, Iterator
+from collections.abc import AsyncIterable, AsyncIterator, Awaitable, Callable, Iterable
+from functools import partial
 from hashlib import new as hash_new
 from inspect import isasyncgenfunction, isawaitable
+from itertools import pairwise
 from os import fstat
 from typing import overload, runtime_checkable, Any, Literal, Protocol, TypeVar
 
@@ -38,6 +40,98 @@ class HashObj(Protocol):
         pass
 
 
+class ChunkedHash(HashObj):
+
+    def __init__(
+        self, 
+        /, 
+        data: Buffer = b"", 
+        alg: str | Callable[[], HashObj] = "md5", 
+        block_size: int = 1 << 20, 
+        sep: bytes = b"", 
+        convert_block: None | Callable[[bytes], bytes] = None, 
+    ):
+        if isinstance(alg, str):
+            alg = partial(hash_new, alg)
+        self.factory = alg
+        self.block_size = block_size
+        self.sep = sep
+        self.convert_block = convert_block
+        self._block_hash_list: list[bytes] = []
+        self._hashobj = alg()
+        self._hashobj_cache = alg()
+        self._at = 0
+        if data:
+            self.update(data)
+
+    @property
+    def count_read(self, /) -> int:
+        return self._at
+
+    def update(self, data: Buffer, /):
+        factory = self.factory
+        block_size = self.block_size
+        block_hash_list = self._block_hash_list
+        push_block = block_hash_list.append
+        hashobj = self._hashobj
+        update = hashobj.update
+        hashobj_cache = self._hashobj_cache
+        m = memoryview(data)
+        if r := self._at % block_size:
+            expect_step = block_size - r
+            b = m[:expect_step]
+            step = len(b)
+            update(b)
+            hashobj_cache.update(b)
+            self._at += step
+            if step < expect_step:
+                return
+            push_block(hashobj_cache.digest())
+            hashobj_cache = self._hashobj_cache = factory()
+        for l, r in pairwise(range(r, len(m) + block_size, block_size)):
+            b = m[l:r]
+            step = len(b)
+            update(b)
+            hashobj_cache.update(b)
+            self._at += step
+            if step < block_size:
+                return
+            push_block(hashobj_cache.digest())
+            hashobj_cache = self._hashobj_cache = factory()
+
+    def digest(self, /) -> bytes:
+        convert_block = self.convert_block
+        if convert_block:
+            block_hash_list = list(map(convert_block, self._block_hash_list))
+        else:
+            block_hash_list = [*self._block_hash_list]
+        if self._at % self.block_size:
+            hashval = self._hashobj_cache.digest()
+            if convert_block:
+                hashval = convert_block(hashval)
+            block_hash_list.append(hashval)
+        hashobj = self.factory()
+        update  = hashobj.update
+        if sep := self.sep:
+            update(block_hash_list[0])
+            for bhash in block_hash_list[1:]:
+                update(sep)
+                update(bhash)
+        else:
+            for bhash in block_hash_list:
+                update(bhash)
+        return hashobj.digest()
+
+    def digest_total(self, /) -> bytes:
+        return self._hashobj.digest()
+
+    def hexdigest(self, /) -> str:
+        return self.digest().hex()
+
+    def hexdigest_total(self, /) -> str:
+        return self._hashobj.hexdigest()
+
+
 @overload
 def make_accumulator(
     func: Callable[[Buffer, T], T], 
@@ -148,7 +242,7 @@ def file_mdigest(
         update = get_digest(digest)
     if hasattr(file, "getbuffer"):
         file = file.getbuffer()
-    b_it: Iterator[Buffer]
+    b_it: Iterable[Buffer]
     if isinstance(file, Buffer):
         file = memoryview(file)[start:stop]
         if not file:
@@ -198,8 +292,9 @@ def file_mdigest(
             b_it = bytes_iter(b_it, stop - start)
     length = 0
     for chunk in b_it:
-        update(chunk)
-        length += (size := len(chunk))
+        m = memoryview(chunk)
+        update(m)
+        length += (size := len(m))
         if callback is not None:
             callback(size)
     return length, results
@@ -329,8 +424,9 @@ async def file_mdigest_async(
             b_it = bytes_async_iter(b_it, stop - start)
     length = 0
     async for chunk in b_it:
-        await update(chunk)
-        length += (size := len(chunk))
+        m = memoryview(chunk)
+        await update(m)
+        length += (size := len(m))
         if callback is not None:
             await callback(size)
     return length, results
